@@ -23,8 +23,8 @@ void lazyfreeFreeObject(void *args[]) {
 void lazyfreeFreeDatabase(void *args[]) {
     kvstore *da1 = args[0];
     kvstore *da2 = args[1];
-    ebuckets oldHfe = args[2];
-    ebDestroy(&oldHfe, &hashExpireBucketsType, NULL);
+    estore *subexpires = args[2];
+    estoreRelease(subexpires);
     size_t numkeys = kvstoreSize(da1);
     kvstoreRelease(da1);
     kvstoreRelease(da2);
@@ -206,12 +206,17 @@ void emptyDbAsync(redisDb *db) {
         flags |= KVSTORE_FREE_EMPTY_DICTS;
     }
     kvstore *oldkeys = db->keys, *oldexpires = db->expires;
-    ebuckets oldHfe = db->hexpires;
+    estore *oldsubexpires = db->subexpires;
     db->keys = kvstoreCreate(&dbDictType, slot_count_bits, flags | KVSTORE_ALLOC_META_KEYS_HIST);
     db->expires = kvstoreCreate(&dbExpiresDictType, slot_count_bits, flags);
-    db->hexpires = ebCreate();
-    atomicIncr(lazyfree_objects, kvstoreSize(oldkeys));
-    bioCreateLazyFreeJob(lazyfreeFreeDatabase, 3, oldkeys, oldexpires, oldHfe);
+    db->subexpires = estoreCreate(&subexpiresBucketsType, slot_count_bits);
+    emptyDbDataAsync(oldkeys, oldexpires, oldsubexpires);
+}
+
+/* Empty a Redis DB data asynchronously. */
+void emptyDbDataAsync(kvstore *keys, kvstore *expires, ebuckets hexpires) {
+    atomicIncr(lazyfree_objects, kvstoreSize(keys));
+    bioCreateLazyFreeJob(lazyfreeFreeDatabase, 3, keys, expires, hexpires);
 }
 
 /* Free the key tracking table.
